#!/usr/bin/env Rscript
#+ setup, cache=FALSE

suppressMessages(require(docopt))

doc <- '
Convert a cuffdiff into a dge-report
Usage: cuffdiff_report.R [options] <cuffdb_directory>

Options:
--constrasts=<tab_delim_table> Table with sample pairs for which dge analysis should be performed
--qcutoff <qcutoff>    Use a q-value cutoff of 0.01 instead of a q-value cutoff [default: 0.01]
--pcutoff <pcutoff>    Override q-value filter and filter by p-value instead
--minfpkm <minfpkm>   Minimal expression in any of the sample to be included in the final result list [default: 1]
'
#--directed           Perform directed dge-steps in a directed manner
#-S                     Dont do general statistics and qc analysis

#print("args ars")
#print(commandArgs(TRUE))

opts <- docopt(doc, commandArgs(TRUE)) ## does not work when spining
## florio_11b: setwd("/lustre/projects/bioinfo/holger/projects/florio_11b/cuffdiff/dge_report"); opts <- docopt(doc, "--constrasts contrasts.txt --pcutoff 0.01 /projects/bioinfo/holger/projects/florio_11b/cuffdiff")
# opts <- docopt(doc, "--undirected  ..")
# opts <- docopt(doc, paste(Sys.getenv("DGE_PARAMS"), ".."))
## todo use commandArgs here!!
#opts <- docopt(doc, paste(Sys.getenv("DGE_PARAMS"), paste(commandArgs(TRUE), collapse=" ")))
#print(opts)

skipQC <<- opts$S
#split_hit_list <- opts$directed
constrasts_file <- opts$constrasts
pcutoff <- if(is.null(opts$pcutoff)) NULL else as.numeric(opts$pcutoff)
qcutoff <- if(is.numeric(pcutoff)) NULL else as.numeric(opts$qcutoff)
if(is.numeric(pcutoff)) opts$qcutoff <- NULL

minFPKM <- as.numeric(opts$minfpkm)


## do basic validation of configuration
stopifnot(is.numeric(qcutoff) | is.numeric(pcutoff))

cuffdb_directory <- normalizePath(opts$cuffdb_directory)

if(is.na(file.info(cuffdb_directory)$isdir)){
    stop(paste("db directory does not exist", doc))
}


devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/master/R/core_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.8/R/ggplot_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/master/R/bio/diffex_commons.R")

#+ results='asis', echo=FALSE
cat('
<link rel="stylesheet" type="text/css" href="http://cdn.datatables.net/1.10.5/css/jquery.dataTables.min.css">
<script type="text/javascript" charset="utf8" src="http://code.jquery.com/jquery-2.1.2.min.js"></script>
<script type="text/javascript" charset="utf8" src="http://cdn.datatables.net/1.10.5/js/jquery.dataTables.min.js"></script>

<script type="text/javascript">
         $(document).ready(function() {
            // alert("test")
             $(".dtable").DataTable();
         } );
</script>
')

require(cummeRbund)
require(knitr)

## reload dplyr to fix namespace issues
unloadNamespace('dplyr'); require(dplyr)
#iris %>% count(Species)

#' Used DGE Analysis Parameters
#vec2df(unlist(opts)) %>% filter(!str_detect(name, "^[<-]")) %>% kable()


########################################################################################################################
#' # Differential Gene Expression Analysis

#' [CuffDiff](http://cufflinks.cbcb.umd.edu/manual.html) and [cummeRbund](http://compbio.mit.edu/cummeRbund/) were used to perform this analysis. For details about how the test for differntial expression works, refer to [Differential analysis of gene regulation at transcript resolution with RNA-seq](http://www.nature.com/nbt/journal/v31/n1/full/nbt.2450.html).


#+ load_db, cache=FALSE, warning=FALSE
isCluster=Sys.getenv("LSF_SERVERDIR")!=""
isBioinfoMountPoint=str_detect(normalizePath(cuffdb_directory), fixed("/mnt/"))
isLocalMountPoint=str_detect(normalizePath(cuffdb_directory), fixed("/Volumes/projects"))

dbTempCopyRequired=isCluster | isBioinfoMountPoint | isLocalMountPoint

if(dbTempCopyRequired){
    ## workaround until sqlite  is fixed on lustre
    tmpdir <- tempfile("cuffdb_dir")
    system(paste("cp -r", cuffdb_directory, tmpdir))
    cuff <- readCufflinks(dir=tmpdir)
}else{
    cuff <- readCufflinks(dir="..")
}

#' The expression tracking reference contained:
cuff

#' Used CuffDiff:
runInfo(cuff) %>% filter(param!="cmd_line") %>% kable()
#' cuffdiff cmd was
#+ results='asis'
cat((runInfo(cuff) %>% filter(param=="cmd_line"))$value %>% gsub('(.{1,80})(,|$|\\s)', '\\1\n', .))

#' ### Samples
replicateInfo <- replicates(cuff) %>%
    mutate(file=basename(file)) %>%
    select(-c(total_mass, norm_mass, internal_scale, external_scale))

replicateInfo %>% kable() # kable("html", table.attr = "class='dtable'", escape=F)

write.delim(replicateInfo, file="replicate_info.txt")
# replicateInfo <- read.delim("replicate_info.txt")
#'  [replicateInfo](replicate_info.txt)

#+ results='asis'
## define or load the contrasts
dbContrasts <- diffData(genes(cuff)) %>% select(sample_1, sample_2) %>% distinct()
if(!is.null(constrasts_file)){
    echo("Using contrasts matrix from: ", basename(constrasts_file))

    contrasts <- read.csv(constrasts_file, header=F) %>% set_names(c("sample_1", "sample_2"))

    ## filter for the direction present in cuffdb
    contrasts <- rbind_list(contrasts, contrasts %>% select(sample_1=sample_2, sample_2=sample_1)) %>%  inner_join(dbContrasts)
}else{
    echo("Comparing all samples against each other")
    contrasts <- diffData(genes(cuff)) %>% select(sample_1, sample_2) %>% distinct()
}

#' The used contrasts model is
contrasts %>% kable()

## download gene info and locations from biomart
geneInfo <- quote({
    martName <- guess_mart(fpkm(genes(cuff))$gene_id)
    mart <- biomaRt::useDataset(martName, mart = biomaRt::useMart("ensembl"))
    biomaRt::getBM(c('ensembl_gene_id', 'external_gene_name', 'description', 'gene_biotype'), mart=mart)
}) %>% cache_it("gene_info")

geneLoci <- quote({
    martName <- guess_mart(fpkm(genes(cuff))$gene_id)
    mart <- biomaRt::useDataset(martName, mart = biomaRt::useMart("ensembl"))
    biomaRt::getBM(c("ensembl_gene_id", "chromosome_name", "start_position", "end_position"), mart=mart)
}) %>% cache_it("gene_loci")
#knitr::opts_knit$set(root.dir = getwd())



#######################################################################################################################
#' ## Count and Expression Score Tables

#' All data has been annotated with ensembl gene information and was exprorted into . Both normalized (FPKM) and raw counts were exported.

fpkmMatrix(genes(cuff)) %>%
    rownames2column("ensembl_gene_id") %>%
    merge(geneInfo, by="ensembl_gene_id", all.x=T) %T>%
    glimpse() %>%
    write.delim(file="gene_fpkms.txt")
# gene_fpkms <- read.delim("gene_fpkms.txt")
#' [Normalized Counts](gene_fpkms.txt)

## export the same but now including replicate information

repFpkmMatrix(genes(cuff)) %>%
    rownames2column("ensembl_gene_id") %>%
    merge(geneInfo, by="ensembl_gene_id", all.x=T) %T>%
    glimpse() %>%
    write.delim(file="gene_fpkms_by_replicate.txt")
# gene_fpkms_by_replicate <- read.delim("gene_fpkms_by_replicate.txt")
#' [Normalized Counts by Replicate](gene_fpkms_by_replicate.txt)

## also export count tables
countMatrix(genes(cuff)) %>%
    rownames2column("ensembl_gene_id") %>%
    merge(geneInfo, by="ensembl_gene_id", all.x=T) %>%
    write.delim(file="gene_counts.txt")
# gene_counts <- read.delim("gene_counts.txt")
#' [Gene Counts](gene_counts.txt)

repCountMatrix(genes(cuff)) %>%
    rownames2column("ensembl_gene_id") %>%
    merge(geneInfo, by="ensembl_gene_id", all.x=T) %>%
    write.delim(file="gene_counts_by_replicate.txt")
# rep_counts_by_replicate <- read.delim("gene_counts_by_replicate.txt")
#' [Gene Counts by Replicate](gene_counts_by_replicate.txt)


#######################################################################################################################
#' ## Quality Control

#' Create some global summary statistic plots in order to get an idea about the general quaniltiy and distribution of the expression data. Descriptions are copied from [Cummerbund Manual](http://compbio.mit.edu/cummeRbund/manual_2_0.html)


#' Several plotting methods are available that allow for quality-control or global analysis of cufflinks data. A good place to begin is to evaluate the quality of the model fitting. Overdispersion is a common problem in RNA-Seq data. As of cufflinks v2.0 mean counts, variance, and dispersion are all emitted, allowing you to visualize the estimated overdispersion for each sample as a quality control measure.
dispersionPlot(genes(cuff)) #+ aes(alpha=0.01)

#' The coefficient of variation (CV) is defined as the ratio of the standard deviation the mean. The squared coefficient of variation is a normalized measure of cross-replicate variability that can be useful for evaluating the quality your RNA-seq data. Differences in CV2 can result in lower numbers of differentially expressed genes due to a higher degree of variability between replicate fpkm estimates.
fpkmSCVPlot(genes(cuff))
#fpkmSCVPlot(genes(cuff), showPool=T)

#' To assess the distributions of FPKM scores across samples, we plot their densities
#csDensity(genes(cuff))
#ggsave2(csBoxplot(genes(cuff))+ ggtitle("fpkm boxplots"))

#csDensity(genes(cuff),features=T)
csBoxplot(genes(cuff),replicates=T)
csBoxplot(genes(cuff))

csScatterMatrix(genes(cuff))

#csVolcano(genes(cuff),"a", "b")
#' For the volcano plot matrix we calculate fold-changes as x/y (ie column/row). For details see [here](http://rpackages.ianhowson.com/bioc/cummeRbund/man/csVolcano.html). Signifcance coloring is based on cuffdiff's internal score an not our hit criterion.
csVolcanoMatrix(genes(cuff))

#' We also plot more detailed scatter plots for just the contrasts for interest. Signifcance coloring is based on cuffdiff's internal score an not our hit criterion.
#csVolcano(genes(cuff),"hESC","Fibroblasts")
csVolcPlots <- alply(contrasts, 1,  splat(function(sample_1, sample_2) {
    csVolcano(genes(cuff), sample_1, sample_2)
 }))

#+ fig.width=16
multiplot(plotlist=csVolcPlots, cols=min(3, length(csVolcPlots)))

#csScatter(genes(cuff),"hESC","Fibroblasts",smooth=T)
#csScatter(genes(cuff), "a", "d", smooth=F) + aes(color=abs(log2(a/b))>2) + scale_colour_manual(values=c("darkgreen", "red"))

#MAplot(genes(cuff), "a", "d")+ aes(color=abs(log2(M))>2)+ ggtitle("MA-plot: a/d")
#MAplot(genes(cuff), "a", "b", useCount=T) #+ aes(color=abs(log2(M))>2)+ ggtitle("MA-plot: Luc/KD")

#csVolcano(genes(cuff),"aRGm", "N") + ggtitle("odd line-shaped fan-out in volcano plot")

## todo consider to add more pretty volcano plots (see stefania's extra plots
#getGenes(cuff,c("ENSMUSG00000097853")) %>%  expressionBarplot()


#######################################################################################################################
#' ## Replicate Correlation and Clustering

#' For most type of experiments we expect replicates to cluster together. Second similar biological conditions should cluster together. As clustering results depend on the method being used, we try a few here to check if we get consistent results.

#' Map replicates into 2-dimensional space to highlight replicate clusters (or the lack of them)
require.auto(edgeR)
plotMDS(repFpkmMatrix(genes(cuff)), top=500, main="top500 MDS")


if(unlen(replicates(cuff)$sample) > 2){  ## >2 only, because MDS will fail with just 2 samples
    MDSplot(genes(cuff)) + ggtitle("mds plot")
}

## todo replace with marta plot
#PCAplot(genes(cuff),"PC1","PC2",replicates=T)

noTextLayer <- function(gg){ gg$layers[[2]] <- NULL;; gg}


## todo use fix scaled here
noTextLayer(csDistHeat(genes(cuff)) +  ggtitle("Sample correlation"))

noTextLayer(csDistHeat(genes(cuff),replicates=T) +  ggtitle("Replicate Correlation")) #+ scale_fill_gradientn(colours=c(low='lightyellow',mid='orange',high='darkred'), limits=c(0,0.15))

csDendro(genes(cuff),replicates=T)

#pdf(paste0(basename(getwd()),"_rep_clustering.pdf"))
#csDendro(genes(cuff),replicates=T)
#dev.off()

#' Also cluster only expressed genes (fpkm >1 for at least one sample)
exprGenes <- repFpkmMatrix(genes(cuff)) %>%
    rownames2column("ensembl_gene_id") %>%
    melt(value.name="fpkm") %>%
    group_by(ensembl_gene_id) %>%
    ## apply log trafo
    mutate(log_fpkm=log10(fpkm+1)) %>%
    ## apply expression filter
    filter(max(log_fpkm)>1) %>%
    dcast( ensembl_gene_id ~ variable, value.var="log_fpkm") %>%
    column2rownames("ensembl_gene_id")

res<-JSdist(makeprobs(exprGenes)) %>% hclust() %>% as.dendrogram()

par(mar=c(10,4,4,4))
plot(res, main="Replicate Clustering of Expressed Genes")


#######################################################################################################################
#' ## Differentially expressed genes

#' Here we are using [cuffdiff](http://cole-trapnell-lab.github.io/cufflinks/cuffdiff) to test for differential expression. Estimate variance-mean dependence in count data from high-throughput sequencing assays and test for differential expression based on a model using the negative binomial distribution.

allDiff <- diffData(genes(cuff)) %>%
    ## just keep pairs of interest
    merge(contrasts) %>%

    ##  discard all genes that are not expressed in any of the cell types (>1)
    filter(gene_id %in% getExpressedGenes(cuff, minFPKM=minFPKM)) %>%
    rename(ensembl_gene_id=gene_id, s2_over_s1_log2fc=log2_fold_change) %>%
    mutate(sample_1_overex=s2_over_s1_log2fc<0)

#+ results='asis'
if(!is.null(qcutoff)){
    echo("Using q-value cutoff of", qcutoff)
    allDiff <- transform(allDiff, is_hit=q_value<=qcutoff)
}else{
    echo("Using p-value cutoff of", pcutoff)
    allDiff <- transform(allDiff, is_hit=p_value<=pcutoff)
}

merge(allDiff, geneInfo, by="ensembl_gene_id", all.x=T) %>% write.delim(file="diff_data.txt")
# allDiff <- read.delim("diff_data.txt")
#'  [Unfiltered CuffDiff Results](diff_data.txt)
#

# Filter for DEGs (differentially expressed genes)
degs <- allDiff %>%
    ## Filter for expressed genes
    filter(is_hit) %>%
    ## filter(significant=="yes") ## original hit-flag provided by cummerbund§
    ## add gene info
    left_join(geneInfo, by="ensembl_gene_id")


#'  DEGs (differentially expressed genes) are contained in the following table
write.delim(degs, file="degs.txt")
# degs <- read.delim("degs.txt")
#' [Differentially Expressed Genes](degs.txt)

#' ### DEG Counts
#with(degs, as.data.frame(table(sample_1, sample_2, sample_1_overex))) %>% filter(Freq >0) %>% kable()
degs %>% count( sample_1, sample_2, sample_1_overex) %>% kable()


#+ fig.height=nrow(contrasts)+2
#ggplot(degs, aes(paste(sample_1, "vs",  sample_2), fill=status)) + geom_bar() +xlab(NULL) + ylab("# DGEs") +ggtitle("DEGs by contrast") + coord_flip()
ggplot(degs, aes(paste(sample_1, "vs",  sample_2), fill=sample_1_overex)) + geom_bar() + xlab(NULL) + ylab("# DGEs") + ggtitle("DEGs by contrast") + coord_flip()


#' DEGs can be browsed in Excel using the exported table or via the embedded table browser. To enable the IGV links, you need to set the port in the IGV preferences to 3334.
kableDegs <- degs
if(nrow(degs>2000)){
    kableDegs <- degs %>% arrange(-q_value) %>% head(1000)
    print("just showing first 1000 most significant hits in interactive hit table!!!! Use Excel to browser to browse the full set")
}

#+ results='asis'
kableDegs %>%
    inner_join(geneLoci) %>%
    mutate(
        ensembl=paste0("<a href='http://www.ensembl.org/Multi/Search/Results?y=0;site=ensembl_all;x=0;page=1;facet_feature_type=Gene;q=",ensembl_gene_id, "'>",ensembl_gene_id, "</a>"),
        igv=paste0("<a href='http://localhost:60151/goto?locus=", chromosome_name,":", start_position, "-", end_position, "'>IGV</a>")
    ) %>%
    select(sample_1, sample_2, value_1, value_2, s2_over_s1_log2fc, p_value, q_value, ensembl, external_gene_name, description, gene_biotype, igv) %>%
    kable("html", table.attr = "class='dtable'", escape=F)

## just needed to restore environment easily
save(degs, file=".degs.RData")
# degs <- local(get(load(".degs.RData")))

#' ### MA Plot
#' Redo MA plots but now including hit overlays

maPlots <- allDiff %>% group_by(sample_1, sample_2) %>% do(gg={ maData <-.
    ## todo why not s2_over_s1_log2fc
    maData %>% ggplot(aes(0.5*log2(value_1*value_2), log2(value_1/value_2), color=is_hit)) +
        geom_point(alpha=0.3) +
        geom_hline(yintercept=0, color="red") +
        ggtitle(with(maData[1,], paste(sample_1, "vs", sample_2)))
}) %$% gg

#+ fig.height=10*ceiling(length(maPlots)/2)
multiplot(plotlist=maPlots, cols=min(2, length(maPlots)))


#' ### Relation of FC and p-values
#' To assess the relation of fold-changes and p-value we plot the data as volcano plots for each contrasts including the number of DEGS according our hit criterion.

hitCounts <- degs %>%
        count(sample_1, sample_2, sample_1_overex) %>%
        rename(hits=n) %>%
        merge(data.frame(sample_1_overex=c(T,F), x_pos=c(-3,3)))

##' For the volcano plot matrix we calculate fold-changes as x/y (ie column/row). For details see [here](http://rpackages.ianhowson.com/bioc/cummeRbund/man/csVolcano.html)
#+ fig.width=16, fig.height=14
#allDiff %>% ggplot(aes(s2_over_s1_log2fc, -log10(p_value), color=is_hit)) +
##    geom_jitter(alpha=0.3, position = position_jitter(height = 0.2)) +
#    geom_point(alpha=0.3) +
##    theme_bw() +
#    xlim(-3.5,3.5) +
#    scale_color_manual(values = c("TRUE"="red", "FALSE"="black")) +
##    ggtitle("Insm1/Ctrl") +
#    ## http://stackoverflow.com/questions/19764968/remove-point-transparency-in-ggplot2-legend
#    guides(colour = guide_legend(override.aes = list(alpha=1))) +
#
#    ## tweak axis labels
#    xlab(expression(log[2]("x/y"))) +
#    ylab(expression(-log[10]("p value"))) +
#
#    ## add hit couts
#    geom_text(aes(label=hits, x=x_pos), y=2, color="red", size=10, data=hitCounts) +
#    facet_grid(sample_1 ~ sample_2)


volcPlots <- allDiff %>% group_by(sample_1, sample_2) %>% do(gg={ maData <-.
    maData %>% ggplot(aes(s2_over_s1_log2fc, -log10(p_value), color=is_hit)) +
    #    geom_jitter(alpha=0.3, position = position_jitter(height = 0.2)) +
        geom_point(alpha=0.3) +
    #    theme_bw() +
        xlim(-3.5,3.5) +
        scale_color_manual(values = c("TRUE"="red", "FALSE"="black")) +
    #    ggtitle("Insm1/Ctrl") +
        ## http://stackoverflow.com/questions/19764968/remove-point-transparency-in-ggplot2-legend
#        guides(colour = guide_legend(override.aes = list(alpha=1))) +
        guides(colour = F) +

        ## tweak axis labels
#        xlab(expression(log[2]("x/y"))) +
        with(maData[1,], xlab(paste0("log2(", sample_2, "/", sample_1, ")"))) +
        ylab(expression(-log[10]("p value"))) +
#        with(maData[1,], ggtitle(paste0(sample_2, " over ", sample_1))) +

        ## add hit couts
        geom_text(aes(label=hits, x=x_pos), y=2.5, color="red", size=10, data=semi_join(hitCounts, maData))
}) %$% gg

#+ fig.height=10*ceiling(length(maPlots)/2)
volcPlots %>% multiplot(plotlist=., cols=min(2, length(.)))


## redefine because changed after v1.8
plotPDF <- function(fileBaseName, expr, ...){ pdf(paste0(fileBaseName, ".pdf"), ...); expr; dev.off(); }
plotPDF("contrasts_volcano", volcPlots %>% multiplot(plotlist=., cols=min(2, length(.))), width=12, height=ceiling(nrow(contrasts)/2)*8 )
#' [volcano PDF](contrasts_volcano.pdf)


########################################################################################################################
#' ## Term enrichment
#+ echo=FALSE

#' This analysis was performed using [David](http://david.abcc.ncifcrf.gov/). The following ontologies were tested: `r paste(ontologies, collapse=', ')`

geneLists <-  degs %>% transmute(ensembl_gene_id, list_id=paste(sample_1, "!=", sample_2))

## also do directed analysis if we have not too many contrasts
if(nrow(contrasts)<3){
    geneLists <- rbind_list(
        geneLists,
        degs %>% filter(sample_1_overex) %>% transmute(ensembl_gene_id, list_id=paste(sample_1, ">", sample_2)),
        degs %>% filter(!sample_1_overex) %>% transmute(ensembl_gene_id, list_id=paste(sample_1, "<", sample_2))
    )
}

## additional overlaps before doing the intersection analysis
geneLists %>% count(list_id)

intersectLists <- function(geneLists, listIdA, listIdB, intersectListId) {
    commonGenes <- setdiff(filter(geneLists, list_id==listIdA)$ensembl_gene_id, filter(geneLists, list_id==listIdB)$ensembl_gene_id)
    data.frame(list_id=intersectListId, ensembl_gene_id=commonGenes)
}

## make this project specific setting
#with(geneLists, as.data.frame(table(list_id))) %$% ac(unique(list_id))
if(any(str_detect(geneLists$list_id, "cyst"))){
    geneLists <- intersectLists(geneLists, "liver != unpolarised", "cyst != unpolarised", "unp_liv & unp_cys") %>% rbind_list(geneLists)
}

geneLists <- distinct(geneLists) ## just precaution in case of multiple evals of statements above


#+ eval=T
enrResults <- quote(geneLists %>% group_by(list_id) %>% do(davidAnnotationChart(.$ensembl_gene_id))) %>%
    cache_it(paste0("enrdata_", digest(geneLists)))

write.delim(enrResults, file="enrResults.txt")
# enrResults <- read.delim("enrResults.txt")
#'  [Enrichment Results](enrResults.txt)

#ac(annoChart$Genes[1]) %>% str_split(", ") %>% unlist() %>% length()

#
#' Because David is not too strigent by default we extract just those terms for which Bonferroni<0.01
sigEnrResults <- subset(enrResults, Bonferroni <0.01)


write.delim(sigEnrResults, file="sigEnrResults.txt")
# sigEnrResults <- read.delim("sigEnrResults.txt")
#'  [Very Significant Terms](sigEnrResults.txt)

#' results='asis'
sigEnrResults %>% select(-Genes) %>% kable("html", table.attr = "class='dtable'", escape=F)

## plot the enrichment results
#sigEnrResults %>% group_by(Category, add=T) %>% do({
#    logPlot <- . %>% ggplot(aes(Term, PValue)) +
#	    geom_bar(stat="identity")+coord_flip() +
#	    xlab("Enriched Terms") +
#	    ggtitle(.$Category[1]) +
#	    scale_y_log10()
#	    print(logPlot)
#})

#+ include=FALSE
warning("dropping levels")
sigEnrResults %<>% mutate(Category=ac(Category)) ## drop unsused level to get consistent color palette

term_category_colors <- create_palette(unique(ac(sigEnrResults$Category)))

term_barplot_files <- sigEnrResults %>%
## chop and pad category names
mutate(Term=str_sub(Term, 1, 70) %>% str_pad(70)) %>%
do({
    enrResultsGrp <- .
    ## DEBUG enrResultsGrp <- sigEnrResults
    label <- enrResultsGrp$list_id[1]

#    warning(paste0("processing terms", paste(ac(unique(enrResultsGrp$Category)), collapse=",")))
    logPlot <- enrResultsGrp %>%
        ## fix factor order
        mutate(Term=reorder(Term, -PValue) %>% reorder(as.integer(as.factor(Category)))) %>%
        ggplot(aes(Term, PValue, fill=Category)) +
        geom_bar(stat="identity")+
        scale_fill_manual(values = term_category_colors, drop=F, name="Ontology") +
        coord_flip() +
        xlab("Enriched Terms") +
        ggtitle(label) +
        scale_y_log10()

    fileNameLabel <- label %>%
        str_replace_all("!=", "ne") %>%
        str_replace_all(">", "gt") %>%
        str_replace_all("<", "lt") %>%
        str_replace_all(fixed("&"), "AND") %>%
        str_replace_all(fixed("|"), "OR") %>%
        str_replace_all(" ", "_")

#        ggsave(paste0("enrichmed_terms__", fileNameLabel, ".pdf"))
#        print(logPlot)

     tmpPng <- paste0("enrterms__", fileNameLabel, ".png")
     ggsave(tmpPng, logPlot, width=10, height = 2+round(nrow(enrResultsGrp)/5), limitsize=FALSE)
     data.frame(file=tmpPng)
})

#+ results="asis"
l_ply(term_barplot_files$file, function(pngFile){ cat(paste0("<img src='", pngFile, "'><br>"))})


########################################################################################################################
#' ## Enriched KEGG Pathways

#' To understand spatio-temporal changes in gene expression better we now overlay enriched kegg pathways with the -log10(q_value) of each contrast. The direction of the expression changes is encoded as color, whereby red indicates that sample_1 is overexpressed. Because we have multiple contrasts of interest, this defines a slice-color barcode for each gene. To relate the barcode to contrasts we define the following slice order:

keggPathways <- sigEnrResults %>%
    filter(Category=="KEGG_PATHWAY") %>%
    separate(Term, c('kegg_pathway_id', 'pathway_description'), sep="\\:", remove=F) %>%
    with(kegg_pathway_id) %>% ac() %>% unique()

require(pathview)
require(png)

# keggPathways <- keggPathways[1]

#if(nrow(keggPathways)==0){
#    echo("no enriched kegg pathways were found in the dataset")
#}else{

#keggOrCode <- "mmu"
keggOrCode <- guess_pathview_species(degs$ensembl_gene_id)

## prepare p-value data
sliceData <- allDiff %>%
    mutate(
        ## define slice labels (needed for casting)
        comparison=paste0(sample_1, "__vs__", sample_2),
        ## create normalized linear score for slice plotting
        sample_1_overex=s2_over_s1_log2fc<0,
        plot_score=-log10(q_value)*ifelse(sample_1_overex, 1, -1)
    ) %>%
    dcast(ensembl_gene_id ~ comparison, value.var="plot_score") %>%
    column2rownames("ensembl_gene_id")

#sliceData %>% head %>% kable()

data.frame(set=names(sliceData)) %>% mutate(slice_index=row_number()) %>% kable()


plot_pathway <- function(pathwayID, overlayData){
#    pathwayID="mmu04015"
#    browser()
#    echo("processing pathway", pathwayID)
    pv.out <- pathview(
            gene.data = overlayData,
            pathway.id = pathwayID,
            species = keggOrCode,
#            out.suffix = pathwayID$kegg.description,
#            out.suffix = pathwayID,
            multi.state = ncol(overlayData)>1,
#            kegg.native=F,
#            node.sum = "mean", # the method name to calculate node summary given that multiple genes or compounds are mapped to it
            limit = list(gene = c(-4,4)),
            gene.idtype="ensembl"
    )

    outfile <- paste0(pathwayID, ".pathview", ifelse(ncol(overlayData)>1, ".multi", ""), ".png")

    ## interactive plotting
#    ima <- readPNG(outfile)
#    plot.new()
#    lim <- par()
#    rasterImage(ima, lim$usr[1], lim$usr[3], lim$usr[2], lim$usr[4])

    pv.out$plotfile=outfile
    pv.out$pathway_id=pathwayID

    return(pv.out)
}

#keggPathways <- c("mmu04976", "mmu04972", "mmu04810", "mmu04520", "mmu04530", "mmu04270", "mmu04015")
#keggPathways <- c("mmu04015")



pathwayPlots <- keggPathways %>% llply(function(pathwayID){
    plot_pathway(pathwayID, sliceData)
})


save(pathwayPlots, file=".pathwayPlots.RData")
# pathwayPlots <- local(get(load("pathwayPlots.RData")))


## prepare tooltips with expression scores
ens2entrez <- quote({
    mart <- biomaRt::useDataset(guess_mart(degs$ensembl_gene_id), mart = biomaRt::useMart("ensembl"))
    biomaRt::getBM(attributes=c('ensembl_gene_id', 'entrezgene', 'external_gene_name'), mart=mart) %>% filter(!is.na(entrezgene))
}) %>% cache_it("ens2entrez") %>% distinct(ensembl_gene_id)
#unlen(ens2entrez$ensembl_gene_id)

toolTipData <- fpkmMatrix(genes(cuff)) %>% rownames2column("ensembl_gene_id") %>% left_join(ens2entrez)

makeTooltip <- function(entrez_id){
    toolTipData %>% filter(entrezgene==entrez_id) %>% dplyr::select(-entrezgene) %>% gather() %$% paste(key, value, sep=": ") %>% paste(collapse="\n") #%>% cat
}
#entrez_id=14679
#entrez_id=34234234234234
#makeTooltip("14679")



#+ results="asis"

## simple non-clickable plots
## http://stackoverflow.com/questions/12588323/r-how-to-extract-values-for-the-same-named-elements-across-multiple-objects-of
#unlist(lapply(pathwayPlots, "[[", "plotfile"))
# unlist(lapply(pathwayPlots, "[[", "plotfile")) %>% l_ply(function(pngFile){ cat(paste0("<img src='", pngFile, "'><br>"))})

#cat("
#<style type='text/css'>
# img {
#   max-width: 100%;
# }
#</style>
#")

## todo add tooltips with additinal info
## http://stackoverflow.com/questions/5478940/how-to-create-a-html-tooltip-on-top-of-image-map

## extended version with clickable links
pathwayPlots %>% l_ply(function(plotData){
    #pngFile="mmu04015.pathview.png"
    #plotData <- pathwayPlots[[1]]
    #plotData <- unlist(pathwayPlots)
#    pathway_id=(basename(pngFile) %>% str_split_fixed("[.]", 2))[,1]
    pngFile <- plotData$plotfile
    pathway_id=plotData$pathway_id

    ## create link for image map
    keggNodes <- plotData$plot.data.gene %>%
        ## remove box offset
        mutate(x=x-22, y=y-8) %>% ## todo does not work for non-gene elements
        mutate(link=paste0("http://www.kegg.jp/dbget-bin/www_bget?", keggOrCode, ":", kegg.names )) %>%
        rowwise() %>% do({ curNode=.; mutate(as.df(curNode), tooltip=makeTooltip(as.integer(curNode$kegg.names)))})

    ## create tooltip by using mapping to


#    "http://www.kegg.jp/dbget-bin/www_bget?mmu:16412

    ## first the image itself
#    cat(paste0("<img usemap='#", pathway_id,"' src='", pngFile, "'><br>"))
    cat(paste0("<p><div style='width: 2000px'><img style='float: left' usemap='#", pathway_id,"' src='", pngFile, "'></div><br>"))
    cat(paste0("<map name='", pathway_id,"'>"))
    #keggNodes %>% rowwise() %>% {curNode=.; cat(curNode$name)}

    #see http://www.html-world.de/180/image-map/
    keggNodes %>% a_ply(1, function(curNode){
        rectDef=with(curNode, paste(x, y, x+width, y+height, sep=","))
#        paste0("<area href='", curNode$link, "' alt='Text' coords='", rectDef , "' shape='rect'>") %>% cat()
        paste0("<area href='", curNode$link, "' title='", curNode$tooltip, "' alt='Text' coords='",rectDef , "' shape='rect'>") %>% cat()

    })
    cat("</map></p><br>")
})

#}

## http://www.w3schools.com/tags/tag_map.asp
## http://stackoverflow.com/questions/358874/how-can-i-use-a-carriage-return-in-a-html-tooltip



#+ include=FALSE
# ########################################################################################################################
# #' ## Protein-Protein Interaction Clusters using String
#
# ## see http://www.bioconductor.org/packages/release/bioc/vignettes/STRINGdb/inst/doc/STRINGdb.pdf
#
# #' We can use the string-db web service to look for protein-protein interactions.
# #' This normally results in a large hairball and many unconnected nodes and so we can look for enriched clusters.
# #' Here we plot using (high confidence interactions) enriched clusters with a node size greater than 5 proteins for our list of candidate genes showing up in at least two replicates
#
# #+ PPI plots , fig.width=30, fig.height=30, fig.retina=2, warning=FALSE, tidy=TRUE
#
# #taxTable <- list(ENSG=9606,  ENSMUSG=10090)
# #taxId <- taxTable[[degs$gene_id[1] %>% ac() %>% str_extract("^([A-z]+)")]]
# #
# #require.auto(STRINGdb)
# #
# #string_db <- STRINGdb$new(score_threshold=400, species=taxId, input_directory="/local2/user_data/brandl/data/stringr") ## score threshold default is 400
# #example1_mapped <- string_db$map( summary_allgids_df_sighit_gt1, "supplier_gene_symbol", species= removeUnmappedRows = TRUE )
# #clustersList <- string_db$get_clusters(example1_mapped$STRING_id)
# #for(network in clustersList)
# #{
# #	if(length(network)>5)
# #	{
# #		string_db$plot_network(network, add_link=FALSE)
# #	}
# #}
#

#+
#' Created `r format(Sys.Date(), format="%B-%d-%Y")` by `r readLines(pipe('whoami'))`, [Scientific Computing Facility](http://www.mpi-cbg.de/facilities/profiles/bioinformatics-and-scientific-computing.html), MPI-CBG